###############################################################################
# ChatBox.pm                                                                  #
###############################################################################
# YaBB: Yet another Bulletin Board                                            #
# Open-Source Community Software for Webmasters                               #
# Version:        YaBB 2.7.00                                                 #
# Distributed by: http://www.yabbforum.com                                    #
# =========================================================================== #
# Copyright (c) 2000-2017 YaBB (www.yabbforum.com) - All Rights Reserved.     #
# Software by:  The YaBB Development Team                                     #
#               with assistance from the YaBB community.                      #
###############################################################################
# Based on Chat CE by Carsten Dalgaard                                        #
###############################################################################
use strict;
no strict qw(refs);
use warnings;
use CGI::Carp qw(fatalsToBrowser);
our $VERSION     = '2.7.00';
our $chatcepmver = 'YaBB 2.7.00 $Revision: 1.6 $';
our ($action);
$action ||= q{};
if ( $action eq 'detailedversion' ) { return 1; }

our (
    %INFO,       %board,     @categoryorder, %catinfo,
    %cat,        $iamfmod,   $annboard,      $iamadmin,
    $uid,        $iamgmod,   %subboard,      %FORM,
    $date,       $username,  $scripturl,     %useraccount,
    %grp_nopost, $abbr_lang, $yymycharset,   $yyhtml_root,
    $usestyle,   $modimgurl, $imagesdir,     $formsession,
    $yymain,     $yytitle,   $yynavigation,  %format,
    $yy_setcookies
);
get_forum_master();
our (
    $enablechat,  $iamguest,   %chatce_txt, $message_term, $part_term,
    $update_freq, $MaxChatLen, $ChatRoom,   %editor_txt,   $removealert,
    $myprinbox,   $mychatpage, $mypart,     $mylink
);

if ( $iamguest || !$enablechat ) { fatal_error('no_access'); }

load_language('Mods/ChatBox');
get_template('Mods/ChatBox');

$message_term ||= 120;
$part_term    ||= 15;
$update_freq  ||= 10;
$MaxChatLen   ||= 300;

my $bigbox_w   = '430px';
my $rightbox_w = '180px';
my $leftbox_w  = '395px';

sub chatCE_open {
    my $chatupdate = $update_freq * 1000;
    my @excllist = split /\,\s/xsm, $ChatRoom;

    my %participant = ();
    if ( -e 'Variables/Mods/chat/chatce.log' ) {
        our ($LOG);
        fopen( 'LOG', '<', 'Variables/Mods/chat/chatce.log' );
        my @part = <$LOG>;
        fclose('LOG');
        foreach my $x (@part) {
            chomp $x;
            my ( $key, $value ) = split /\t/xsm, $x;
            if ($key) {
                $participant{"$key"} = $value;
            }
        }
    }

    my $defselected = q{};
    my $actroom     = 'chatcedefault';
    my $actname     = $chatce_txt{'chatdefault'};
    if ( $INFO{'room'} && $INFO{'room'} ne q{} ) {
        $actroom = $INFO{'room'};
    }
    if ( $actroom eq 'chatcedefault' ) {
        $actname     = $chatce_txt{'chatdefault'};
        $defselected = q~ selected="selected"~;
    }
    else {
        ( $actname, undef, undef ) = @{ $board{$actroom} };
    }

    my $catcptions = qq~    <optgroup label="$chatce_txt{'chatroom'}">
        <option value="chatcedefault"$defselected>$chatce_txt{'chatdefault'}</option>
    </optgroup>~;

    my $boardcount = 0;

    foreach my $catid (@categoryorder) {
        my ( $catname, $catperms, $catallowcol, $catimage ) =
          @{ $catinfo{$catid} };
        to_chars($catname);
        my $cataccess = cat_access($catperms);
        if ( !$cataccess ) { next; }
        my $testcatid = $catid . '_cat';
        my $usecat    = 1;
        for (@excllist) {
            if ( $_ eq $testcatid ) { $usecat = 0; }
        }
        if ( !$usecat ) { next; }
        my $firstboard = 1;
        my $isoptgroup = 0;
        my @bdlist     = @{ $cat{$catid} };
        for my $curboard (@bdlist) {
            my $useboard = 1;
            for (@excllist) {
                if ( $_ eq $curboard ) { $useboard = 0; }
            }
            if ( !$useboard ) { next; }
            if (   $annboard eq $curboard
                && !$iamadmin
                && !$iamgmod
                && !$iamfmod )
            {
                next;
            }
            my ( $boardname, $boardperms, $boardview ) = @{ $board{$curboard} };
            my $access = access_check( $curboard, q{}, $boardperms );
            if ( !$iamadmin && $access ne 'granted' && $boardview != 1 ) {
                next;
            }
            if ( ${ $uid . $curboard }{'brdpassw'} ) { next; }
            if ($firstboard) {
                $catcptions .= qq~<optgroup label="$catname">~;
                $isoptgroup = 1;
            }
            my $selected = q{};
            if ( $curboard eq $actroom ) {
                $selected = q~ selected="selected"~;
            }
            $catcptions .=
              qq~<option value="$curboard"$selected>$boardname</option>\n~;
            $firstboard = 0;
            $boardcount++;
            if ( $subboard{$curboard} ) {
                my @bdsublist = @{ $subboard{$curboard} };
                for my $sbcurboard (@bdsublist) {
                    for (@excllist) {
                        if ( $_ eq $sbcurboard ) { $useboard = 0; }
                    }
                    if ( !$useboard ) { next; }
                    my $bdsubselected = q{};
                    my ( $subboardname, $subboardperms, $subboardview ) =
                      @{ $board{$sbcurboard} };
                    my $subaccess =
                      access_check( $sbcurboard, q{}, $subboardperms );
                    if (  !$iamadmin
                        && $subaccess ne 'granted'
                        && $subboardview != 1 )
                    {
                        next;
                    }
                    if ( ${ $uid . $sbcurboard }{'brdpassw'} ) { next; }
                    for (@excllist) {
                        if ( $sbcurboard eq $actroom ) {
                            $bdsubselected = q~ selected="selected"~;
                        }
                    }
                    $catcptions .=
qq~<option value="$sbcurboard"$bdsubselected>&nbsp; &nbsp;-$subboardname</option>\n~;
                }
            }
        }
        if ($isoptgroup) { $catcptions .= q~</optgroup>~; }
    }

    if ( $FORM{'tmpmessage'} ) {
        my $tmpmessage = $FORM{'tmpmessage'};
        $tmpmessage =~ s/[\r\n]//igxsm;
        $tmpmessage =~ s/(\<a href=)(.*?)(\>)/$1$2 target="_blank"$3/igsm;
        $tmpmessage =~ s/&nbsp;/ /igxsm;
        $tmpmessage =~ s/\Qfont color="\E/span style="color:/gxsm;
        $tmpmessage =~ s/\Qfont face="\E/span style="font-family:/gxsm;
        $tmpmessage =~ s/\/font/\/span/gxsm;
        to_html($tmpmessage);

        my %shouts = ();
        if ( -e "Variables/Mods/chat/$actroom.chat" ) {
            our ($CHAT);
            fopen( 'CHAT', '<', "Variables/Mods/chat/$actroom.chat" );
            my @shout = <$CHAT>;
            fclose('CHAT');
            foreach my $x (@shout) {
                chomp $x;
                my ( $key, $value ) = split /\t/xsm, $x;
                if ($key) {
                    $shouts{"$key"} = $value;
                }
            }
        }

        $shouts{$date} = "$username|$tmpmessage";
        our ($CHAT);
        fopen( 'CHAT', '>', "Variables/Mods/chat/$actroom.chat" );
        print {$CHAT} map { "$_\t$shouts{$_}\n" } keys %shouts
          or croak "cannot print to $actroom.chat";
        fclose('CHAT');
    }

    my $iammod     = 0;
    my %moderators = ();
    my $showmods   = q{};
    if ( ${ $uid . $actroom }{'mods'} ) {
        for my $curuser ( split /\//xsm, ${ $uid . $actroom }{'mods'} ) {
            if ( $username eq $curuser ) { $iammod = 1; }
            if ( !${ $uid . $curuser }{'password'} ) { load_user($curuser); }
            $moderators{$curuser} = $curuser;
            $showmods .=
qq~<a href="$scripturl?action=viewprofile;username=$useraccount{$curuser}" target="_top">${ $uid . $curuser }{'realname'}</a>, ~;
        }
        if ( keys %moderators == 1 ) {
            $showmods = qq~<b>$chatce_txt{'moderator'}:</b><br />$showmods~;
        }
        elsif ( keys %moderators != 0 ) {
            $showmods = qq~<b>$chatce_txt{'moderators'}:</b><br />$showmods~;
        }
        $showmods =~ s/,\s\Z//xsm;
        undef %moderators;
    }

    if ( !${ $uid . $username }{'password'} ) { load_user($username); }

    my %moderatorgroups = ();
    my $showmodgroups   = q{};
    if ( ${ $uid . $actroom }{'modgroups'} ) {
        for my $curgroup ( split /\//xsm, ${ $uid . $actroom }{'modgroups'} ) {
            if ( ${ $uid . $username }{'position'} eq $curgroup ) {
                $iammod = 2;
            }
            foreach ( split /,/xsm, ${ $uid . $username }{'addgroups'} ) {
                if ( $_ eq $curgroup ) { $iammod = 2; last; }
            }
            my ( $thismodgrp, undef ) = @{ $grp_nopost{$curgroup} };
            $moderatorgroups{$curgroup} = $thismodgrp;
            $showmodgroups .= qq~$thismodgrp, ~;
        }

        if ( scalar keys %moderatorgroups == 1 ) {
            $showmodgroups =
              qq~<b>$chatce_txt{'modgroup'}:</b><br />$showmodgroups~;
        }
        elsif ( scalar keys %moderatorgroups != 0 ) {
            $showmodgroups =
              qq~<b>$chatce_txt{'modgroups'}:</b><br />$showmodgroups~;
        }
        $showmodgroups =~ s/,\s\Z//xsm;
        undef %moderatorgroups;
    }
    if ( $showmodgroups eq q{} && $showmods eq q{} ) {
        $showmodgroups = q~<br />~;
    }
    if ( $showmodgroups ne q{} && $showmods ne q{} ) { $showmods .= q~<br />~; }

    if ($iamadmin) { $iammod = 4; }
    if ($iamgmod)  { $iammod = 3; }
    if ($iamfmod)  { $iammod = 5; }

    $participant{$username} = qq~$date|$actroom|$iammod~;

    our ($LOG);
    fopen( 'LOG', '>', 'Variables/Mods/chat/chatce.log' );
    print {$LOG} map { "$_\t$participant{$_}\n" } keys %participant
      or croak 'cannot print to chatce.log';
    fclose('LOG');
    undef %participant;

    my %textdecor = (
        'a' => "bold.png|bold|formatText(this.id)|$editor_txt{'19'}",
        'b' => "italicize.png|italic|formatText(this.id)|$editor_txt{'20'}",
        'c' => "underline.png|underline|formatText(this.id)|$editor_txt{'21'}",
        'd' => "strike.png|strikethrough|formatText(this.id)|$editor_txt{'22'}",
        'e' => "sup.png|superscript|formatText(this.id)|$editor_txt{'10'}",
        'f' => "sub.png|subscript|formatText(this.id)|$editor_txt{'11'}",
    );
    my %boxlist1 = (
        'a' => qq~me.png|me|insertMe()|$editor_txt{'14'}~,
        'b' =>
qq~url.png|insertlin|newLinkWindow("$scripturl?action=createlink;task=hyper","60","460");|$chatce_txt{'hyperlink'}~,
        'c' =>
qq~email2.png|insertemail|newLinkWindow("$scripturl?action=createlink;task=email","60","460");|$editor_txt{'3'}~,
    );
    my $boxlist1   = boxlist(%boxlist1);
    my $textdecor  = boxlist(%textdecor);
    my $my_smilies = mysmilies_list();

    my $prinbrd = q{};
    if ($boardcount) {
        $prinbrd =
qq~<div class="small" style="width:$rightbox_w; font-style: italic; margin-top: 5px; margin-left: auto; margin-right: auto;">
                <select name="ChatRoom" id="ChatRoom" onchange="if(this.options[this.selectedIndex].value) roomUpdate(this.options[this.selectedIndex].value)" style="width: 180px; font-size: 10px; vertical-align: top;">
                    $catcptions
                </select>
                </div>
                <div class="small" style="width:$rightbox_w; font-style: italic; margin: 5px auto;">
                    $showmods $showmodgroups
                </div>~;
    }
    our $prinbox = $myprinbox;
    $prinbox =~ s/{prinbrd}/$prinbrd/gxsm;
    $prinbox =~ s/{textdecor}/$textdecor/gxsm;
    $prinbox =~ s/{boxlist1}/$boxlist1/gxsm;
    $prinbox =~ s/{my_smilies}/$my_smilies/gxsm;
    $prinbox =~ s/{leftbox_w}/$leftbox_w/gxsm;
    $prinbox =~ s/{bigbox_w}/$bigbox_w/gxsm;
    $prinbox =~ s/{rightbox_w}/$rightbox_w/gxsm;
    $prinbox =~ s/{formsession}/$formsession/gxsm;
    $prinbox =~ s/{realname}/${ $uid . $username }{'realname'}/gxsm;
    $prinbox =~ s/{actroom}/$actroom/gxsm;
    $prinbox =~ s/{actname}/$actname/gxsm;
    $prinbox =~ s/{chatupdate}/$chatupdate/gxsm;

    print_output_header();
    our $output = $prinbox;
    print_html_output_and_finish();
    return;
}

sub chatCE {
    my $theroom = 'chatcedefault';
    if ( $INFO{'room'} ) { $theroom = $INFO{'room'}; }

    $yymain .= qq~
    <div class="bordercolor" style="width: 630px; padding: 0px 1px 1px 1px; margin-left: auto; margin-right: auto;">
        <iframe id="chatcemain" style="width: 630px; height: 540px; padding: 0px; font-size: 12px; border:0; overflow: auto"></iframe>
    </div>
    <script type="text/javascript">
    var theroom = '$theroom';
    document.getElementById('chatcemain').src="$scripturl?action=cdchatopen;room=" + theroom;
    </script>
    ~;

    $yynavigation = qq~&rsaquo; $chatce_txt{'chatbox'}~;
    $yytitle      = $chatce_txt{'chatbox'};
    template();
    return;
}

sub UpdateChat {
    my $thisroom = $INFO{'room'};
    my %shouts   = ();
    if ( -e "Variables/Mods/chat/$thisroom.chat" ) {
        our ($CHAT);
        fopen( 'CHAT', '<', "Variables/Mods/chat/$thisroom.chat" );
        my @shout = <$CHAT>;
        fclose('CHAT');
        foreach my $x (@shout) {
            chomp $x;
            my ( $key, $value ) = split /\t/xsm, $x;
            if ($key) {
                $shouts{"$key"} = $value;
            }
        }
        if ( $INFO{'delete'} ) {
            my $delthis = $INFO{'delete'};
            delete $shouts{$delthis};
        }
    }

    my $iammodhere = 0;
    if ( ${ $uid . $thisroom }{'mods'} ) {
        for my $curuser ( split /\//xsm, ${ $uid . $thisroom }{'mods'} ) {
            if ( $username eq $curuser ) { $iammodhere = 1; }
        }
    }
    if ( ${ $uid . $thisroom }{'modgroups'} ) {
        for my $curgroup ( split /\//xsm, ${ $uid . $thisroom }{'modgroups'} ) {
            if ( ${ $uid . $username }{'position'} eq $curgroup ) {
                $iammodhere = 1;
            }
            for ( split /,/xsm, ${ $uid . $username }{'addgroups'} ) {
                if ( $_ eq $curgroup ) { $iammodhere = 1; last; }
            }
        }
    }

    load_censor_list();
    my $toold = $message_term * 60;

    my $rem_alert = q{};
    if ($removealert) {
        $rem_alert = qq~ onclick="return confirm('$chatce_txt{'remove'}');"~;
    }

    my $actcounter = 0;
    my $page_chat  = q{};
    for my $key ( sort keys %shouts ) {
        if ( $key < ( $date - $toold ) ) {
            delete $shouts{$key};
            next;
        }

        my $deletech = q{};
        if ( $iamadmin || $iamgmod || $iamfmod || $iammodhere ) {
            $deletech =
              qq~    <div class="small" style="float: left; width: 20px;">
    <a href="$scripturl?action=cdupdatechat;room=$thisroom;delete=$key"$rem_alert><img src='$imagesdir/im_delete.gif' alt="$chatce_txt{'remove'}" title="$chatce_txt{'remove'}" /></a>
    </div>~;
        }

        my ( $tmpuser, $tmpshout ) = split /[|]/xsm, $shouts{$key};
        from_html($tmpshout);
        $tmpshout = do_censor($tmpshout);
        my $thistime = timeformat($key);

        if ( !${ $uid . $tmpuser }{'password'} ) { load_user($tmpuser); }
        my $theuser = $format{$tmpuser};
        $page_chat .= qq~
    <div class="small" style="float: left; width: 356px; padding: 4px 0px 0px 4px;">&#8226; $thistime - $theuser</div>$deletech
    <div style="float: left; width: 370px; padding: 2px 4px 4px 8px; border-bottom: 1px dotted gray;">$tmpshout</div>~;
        $actcounter++;
    }

    our $chatpage = $mychatpage;
    $chatpage =~ s/{page_chat}/$page_chat/gxsm;

    our ($CHAT);
    fopen( 'CHAT', '>', "Variables/Mods/chat/$thisroom.chat" );
    print {$CHAT} map { "$_\t$shouts{$_}\n" } keys %shouts
      or croak "cannot print to $thisroom.chat";
    fclose('CHAT');
    undef %shouts;

    print_output_header();
    our $output = $chatpage;
    print_html_output_and_finish();
    return;
}

sub UpdateParticipants {
    my $thisroom    = $INFO{'room'};
    my %participant = ();
    if ( -e 'Variables/Mods/chat/chatce.log' ) {
        our ($LOG);
        fopen( 'LOG', '<', 'Variables/Mods/chat/chatce.log' );
        my @part = <$LOG>;
        fclose('LOG');
        foreach my $x (@part) {
            chomp $x;
            my ( $key, $value ) = split /\t/xsm, $x;
            if ($key) {
                $participant{"$key"} = $value;
            }
        }
    }
    my $toold  = $part_term * 60;
    my $getmem = q{};
    foreach my $thisuser ( sort keys %participant ) {
        my ( $ontime, $onroom, $iammodhere ) = split /[|]/xsm,
          $participant{$thisuser};
        if ( $ontime < ( $date - $toold ) ) {
            delete $participant{$thisuser};
            next;
        }
        my (%moderators);
        if ( $onroom ne $thisroom ) { next; }
        if ( $iammodhere == 1 ) { $moderators{$thisuser} = $thisuser; }
        if ( $username eq $thisuser ) { load_miniuser($thisuser); }
        else {
            if ( !${ $uid . $thisuser }{'password'} ) { load_user($thisuser); }
        }
        if ( !$iamadmin && !$iamgmod && ${ $uid . $thisuser }{'stealth'} ) {
            next;
        }
        my $theuser =
          $format{$thisuser}
          . ( ${ $uid . $thisuser }{'stealth'} ? q{*} : q{} );
        my $memstat = q~&#8226;~;
        if ( $iammodhere == 1 || $iammodhere == 2 ) {
            $memstat =
qq~<img src='$imagesdir/starsilver.png' alt="$chatce_txt{'moderator'}" title="$chatce_txt{'moderator'}" style="margin-left: 1px;" />~;
        }
        elsif ( $iammodhere == 3 ) {
            $memstat =
qq~<img src='$imagesdir/stargold.png' alt="$chatce_txt{'gmod'}" title="$chatce_txt{'gmod'}" />~;
        }
        elsif ( $iammodhere == 4 ) {
            $memstat =
qq~<img src='$imagesdir/stargold.png' alt="$chatce_txt{'admin'}" title="$chatce_txt{'admin'}" />~;
        }
        elsif ( $iammodhere == 5 ) {
            $memstat =
qq~<img src='$imagesdir/starsilver.png' alt="$chatce_txt{'fmod'}" title="$chatce_txt{'fmod'}" />~;
        }
        else { $memstat = q~&#8226;~; }
        $getmem .=
qq~    <div style="float: left; width:$rightbox_w; text-align: left; padding: 0;">
        <span class="small" style="float: left; width: 18px; text-align: center; vertical-align: middle;">$memstat</span>
        <span class="small" style="float: left; padding-left: 4px; vertical-align: middle;">$theuser</span><br />
    </div><br style="clear:both" />~;
    }
    our ($LOG);
    fopen( 'LOG', '>', 'Variables/Mods/chat/chatce.log' );
    print {$LOG} map { "$_\t$participant{$_}\n" } keys %participant
      or croak 'cannot print to chatce.log';
    fclose('LOG');
    undef %participant;

    our $part = $mypart;
    $part =~ s/{getmem}/$getmem/gxsm;

    print_output_header();
    our $output = $part;
    print_html_output_and_finish();
    return;
}

sub InsertLink {
    my $linktype  = $INFO{'task'};
    my $linktitle = qq~$chatce_txt{'hyperlink'}~;
    my $linkttext = q~http://~;
    if ( $linktype eq 'email' ) {
        $linktitle = qq~$editor_txt{'3'}~;
        $linkttext = q~mailto:~;
    }

    our $link = $mylink;
    $link =~ s/{linktitle}/$linktitle/gxsm;
    $link =~ s/{linkttext}/$linkttext/gxsm;

    print_output_header();
    our $output = $link;
    print_html_output_and_finish();
    return;
}

sub ColorPicker {
    my $picktask   = $INFO{'task'};
    my $titleextra = q~Font Color~;
    if   ( $picktask eq 'HiliteColor' ) { $titleextra = q~Background Color~; }
    else                                { $titleextra = q~Font Color~; }
    my $browser = $ENV{'HTTP_USER_AGENT'};
    if ( $picktask eq 'HiliteColor' && $browser =~ m/msie/ixsm ) {
        $picktask = q~BackColor~;
    }
    our $color = <<"COLOR";
<!DOCTYPE html>
<html lang="$abbr_lang">
<html>
<head>
<title>$titleextra Palette</title>
<meta http-equiv="Content-Type" content="text/html; charset=$yymycharset" />
<link rel="stylesheet" href="$yyhtml_root/Templates/Forum/$usestyle.css" type="text/css" />
<script type="text/javascript">
var picktask = '$picktask';
function tohex(i) {
    a2 = ''
    ihex = hexQuot(i);
    idiff = eval(i + '-(' + ihex + '*16)')
    a2 = itohex(idiff) + a2;
    while( ihex >= 16) {
        itmp = hexQuot(ihex);
        idiff = eval(ihex + '-(' + itmp + '*16)');
        a2 = itohex(idiff) + a2;
        ihex = itmp;
    }
    a1 = itohex(ihex);
    return a1 + a2 ;
}

function hexQuot(i) {
    return Math.floor(eval(i +'/16'));
}

function itohex(i) {
    if( i === 0) { aa = '0' }
    else { if( i == 1 ) { aa = '1' }
    else { if( i == 2 ) { aa = '2' }
    else { if( i == 3 ) { aa = '3' }
    else { if( i == 4 ) { aa = '4' }
    else { if( i == 5 ) { aa = '5' }
    else { if( i == 6 ) { aa = '6' }
    else { if( i == 7 ) { aa = '7' }
    else { if( i == 8 ) { aa = '8' }
    else { if( i == 9 ) { aa = '9' }
    else { if( i == 10) { aa = 'a' }
    else { if( i == 11) { aa = 'b' }
    else { if( i == 12) { aa = 'c' }
    else { if( i == 13) { aa = 'd' }
    else { if( i == 14) { aa = 'e' }
    else { if( i == 15) { aa = 'f' }
    }}}}}}}}}}}}}}}
    return aa
}

function Pickshowcolor(color) {
    if ( c=color.match(/rgb\\((\\d+?)\\, (\\d+?)\\, (\\d+?)\\)/i) ) {
        var rhex = tohex(c[1]);
        var ghex = tohex(c[2]);
        var bhex = tohex(c[3]);
        var newcolor = '#'+rhex+ghex+bhex;
    }
    else {
        var newcolor = color;
    }
    window.opener.document.getElementById('wysmessage').contentWindow.document.execCommand(picktask, false, newcolor);
    window.close();
}

</script>
</head>
<body style="min-width: 300px;">
<div class="windowbg" style="position: absolute; top: 0px; left: 0px; width: 302px; height: 308px; border: 1px black outset;">
    <div style="position: relative; top: 4px; left: 5px; width: 289px; height: 209px; padding-left: 1px; padding-top: 1px; border: 0px; background-color: black;">
COLOR

    my $selcolor = q{};
    for my $z ( 0 .. 255 ) {
        if ( $z % 51 == 0 ) {
            my $c1 = sprintf '%02x', $z;
            for my $y ( 0 .. 255 ) {
                if ( $y % 51 == 0 ) {
                    my $c2 = sprintf '%02x', $y;
                    for my $x ( 0 .. 255 ) {
                        if ( $x % 51 == 0 ) {
                            my $c3 = sprintf '%02x', $x;
                            $selcolor = qq~#$c3$c2$c1~;
                            $color .=
qq~<span title="$selcolor" style="float: left; background-color: $selcolor; width: 15px; height: 15px; margin-right: 1px; margin-bottom: 1px; border: 0px; font-size: 5px; cursor: pointer;" onclick="Pickshowcolor('$selcolor')">&nbsp;</span>~;
                        }
                    }
                }
            }
        }
    }
    $color .= qq~
        <span class="deftpalchat" style="background-color: #000000;" onclick="Pickshowcolor('#000000')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #222222;" onclick="Pickshowcolor('#222222')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #333333;" onclick="Pickshowcolor('#333333')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #444444;" onclick="Pickshowcolor('#444444')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #555555;" onclick="Pickshowcolor('#555555')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #666666;" onclick="Pickshowcolor('#666666')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #777777;" onclick="Pickshowcolor('#777777')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #888888;" onclick="Pickshowcolor('#888888')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #aaaaaa;" onclick="Pickshowcolor('#aaaaaa')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #bbbbbb;" onclick="Pickshowcolor('#bbbbbb')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #cccccc;" onclick="Pickshowcolor('#cccccc')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #dddddd;" onclick="Pickshowcolor('#dddddd')">&nbsp;</span>
        <span class="deftpalchat" style="background-color: #eeeeee;" onclick="Pickshowcolor('#eeeeee')">&nbsp;</span>
    </div>
    <div style="position: relative; top: 9px; left: 5px; width: 289px; height: 17px; border: 1px black solid;">
        <span name="viewcolor" id="viewcolor" style="float: left; background-color: $selcolor; width: 192px; height: 17px; border-right: 1px black solid; font-size: 5px; cursor: pointer;" onclick="Pickshowcolor(this.style.backgroundColor)">&nbsp;</span>
        <span style="float: right; width: 72px; height: 15px;">
            <input class="windowbg" name="viewcode" id="viewcode" type="text" style="width: 70px; font-size: 11px; border: 0px; display: inline;" readonly="readonly" />
        </span>
    </div>
    <div class="catbg" style="position: relative; top: 15px; left: 10px; width: 277px; height: 56px; border-width: 1px; border-style: outset;">
        <img src="$imagesdir/knapbagrms01.gif" style="position: absolute; top: 0px; left: 0px; z-index: 1; width: 275px; height: 16px;" />
        <img src="$imagesdir/knapred.gif" id="knapImg1" class="skyd" style="position: absolute; left: 4px; top: 2px; cursor: pointer; z-index: 2; width: 13px; height: 15px;" />
        <img src="$imagesdir/knapbagrms01.gif" style="position: absolute; top: 16px; left: 0px; z-index: 1; width: 275px; height: 16px;" />
        <img src="$imagesdir/knapgreen.gif" id="knapImg2" class="skyd" style="position: absolute; left: 4px; top: 18px; cursor: pointer; z-index: 2; width: 13px; height: 15px;" />
        <img src="$imagesdir/knapbagrms01.gif" style="position: absolute; top: 32px; left: 0px; z-index: 1; width: 275px; height: 16px;" />
        <img src="$imagesdir/knapblue.gif" id="knapImg3" class="skyd" style="position: absolute; left: 4px; top: 34px; cursor: pointer; z-index: 2; width: 13px; height: 15px;" />
    </div>
</div>
<script src="$yyhtml_root/palette.js" type="text/javascript"></script>
</body>
</html>~;
    print_output_header();
    our $output = $color;
    print_html_output_and_finish();
    return;
}

sub boxlist {
    my %list    = @_;
    my $hand    = q~class='vtop cursor' style='height:22px; width:23px;'~;
    my $boxlist = q{};
    my $w       = 0;
    for my $i ( sort keys %list ) {
        my ( $img, $id, $onclick, $alt ) = split /[|]/xsm, $list{$i};
        $boxlist .=
qq~<span class="ubbcbutton ubbcbuttonback"><img src='$yyhtml_root/UBBCbuttons/$img' id='$id' onclick='$onclick' $hand alt='$alt' title='$alt' /></span>\n~;
    }
    return $boxlist;
}

sub mysmilies_list {
    my %smiley_bar = (
        'a' => "smiley.gif|smiley|$editor_txt{'101'}",
        'b' => "wink.gif|wink|$editor_txt{'102'}",
        'c' => "cheesy.gif|cheesy|$editor_txt{'103'}",
        'd' => "grin.gif|grin|$editor_txt{'104'}",
        'e' => "angry.gif|angry|$editor_txt{'105'}",
        'f' => "sad.gif|sad|$editor_txt{'106'}",
        'g' => "shocked.gif|shocked|$editor_txt{'107'}",
        'h' => "cool.gif|cool|$editor_txt{'108'}",
        'i' => "huh.gif|huh|$editor_txt{'109'}",
        'j' => "rolleyes.gif|rolleyes|$editor_txt{'110'}",
        'k' => "tongue.gif|tongue|$editor_txt{'111'}",
        'l' => "embarassed.gif|embarassed|$editor_txt{'112'}",
        'm' => "lipsrsealed.gif|lipsrsealed|$editor_txt{'113'}",
        'n' => "undecided.gif|undecided|$editor_txt{'114'}",
        'o' => "kiss.gif|kiss|$editor_txt{'115'}",
        'p' => "cry.gif|cry|$editor_txt{'116'}",
    );

    my $hand =
      q~class='bottom cursor' style='margin:1px;' height='18' width='18'~;
    my $smilies_list = q{};
    for my $i ( sort keys %smiley_bar ) {
        my ( $img, $id, $alt ) = split /[|]/xsm, $smiley_bar{$i};
        $smilies_list .=
qq~<img src='$yyhtml_root/Smilies/$img' id='$id' onclick='insertSmiley(this.src, this.title);' $hand alt='$alt' title='$alt' />\n~;
    }

    return $smilies_list;
}

1;
